/*
 *
 *                 #####    #####   ######  ######  ###   ###
 *               ##   ##  ##   ##  ##      ##      ## ### ##
 *              ##   ##  ##   ##  ####    ####    ##  #  ##
 *             ##   ##  ##   ##  ##      ##      ##     ##
 *            ##   ##  ##   ##  ##      ##      ##     ##
 *            #####    #####   ##      ######  ##     ##
 *
 *
 *             OOFEM : Object Oriented Finite Element Code
 *
 *               Copyright (C) 1993 - 2013   Borek Patzak
 *
 *
 *
 *       Czech Technical University, Faculty of Civil Engineering,
 *   Department of Structural Mechanics, 166 29 Prague, Czech Republic
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef idm1sla_h
#define idm1sla_h

#include "idm1.h"
#include "slamaterialextensioninterface.h"

///@name Input fields for IsotropicDamageMaterial1Sla
//@{
#define _IFT_IsotropicDamageMaterial1Sla_Name "idm1sla"
//@}

namespace oofem {

/**
 * This class implements an extension of associated Material Status to IsotropicDamageMaterial1.
 *
 */
class IsotropicDamageMaterial1StatusSla : public IsotropicDamageMaterial1Status, public SlaMaterialStatusExtensionInterface
{
public:
    /// Constructor
    IsotropicDamageMaterial1StatusSla(int n, Domain * d, GaussPoint * g) : IsotropicDamageMaterial1Status(n, d, g),
                                                                           SlaMaterialStatusExtensionInterface(n, d, g, StructuralMaterial::giveSizeOfVoigtSymVector(g->giveMaterialMode()), 1) {}
    /// Destructor
    virtual ~IsotropicDamageMaterial1StatusSla() { }

    virtual const char *giveClassName() const { return "IsotropicDamageMaterial1StatusSla"; }
    virtual void printOutputAt(FILE *file, TimeStep *tStep);

};

/**
 * This class implements an extension of simple local isotropic damage model for concrete in tension.
 * A model is based on isotropic damage concept updated for sequential analysis
 */
class IsotropicDamageMaterial1Sla : public IsotropicDamageMaterial1, public SlaMaterialExtensionInterface
{

public:

    /// Constructor
    IsotropicDamageMaterial1Sla(int n, Domain * d);
    /// Destructor
    virtual ~IsotropicDamageMaterial1Sla(){}

    // identification and auxiliary functions
    virtual const char *giveClassName() const { return "IsotropicDamageMaterial1Sla"; }
    virtual const char *giveInputRecordName() const { return _IFT_IsotropicDamageMaterial1Sla_Name; }
    virtual IRResultType initializeFrom(InputRecord *ir);
    virtual void giveRealStressVector(FloatArray &answer, GaussPoint *gp,
                                      const FloatArray &totalStrain,
                                      TimeStep *tStep);


    virtual MaterialStatus *CreateStatus(GaussPoint *gp) const;
    virtual MaterialStatus *giveStatus(GaussPoint *gp) const;

    // vorel - extension
    virtual double computeLambda(GaussPoint *gp, FloatArray &totalStrain, TimeStep *tStep, double minLambda = INFINITY);
    virtual void initSLA(GaussPoint *gp, FloatArray &totalStrain, TimeStep *tStep);
    virtual void updateHistoryParameters(GaussPoint *gp, FloatArray &totalStrain, TimeStep *tStep, bool activeDamage);

};
} // end namespace oofem
#endif // idm1sla_h
