/*
 *
 *                 #####    #####   ######  ######  ###   ###
 *               ##   ##  ##   ##  ##      ##      ## ### ##
 *              ##   ##  ##   ##  ####    ####    ##  #  ##
 *             ##   ##  ##   ##  ##      ##      ##     ##
 *            ##   ##  ##   ##  ##      ##      ##     ##
 *            #####    #####   ##      ######  ##     ##
 *
 *
 *             OOFEM : Object Oriented Finite Element Code
 *
 *               Copyright (C) 1993 - 2013   Borek Patzak
 *
 *
 *
 *       Czech Technical University, Faculty of Civil Engineering,
 *   Department of Structural Mechanics, 166 29 Prague, Czech Republic
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef intmatisodamagetablesla_h
#define intmatisodamagetablesla_h


#include "intmatisodamagetable.h"
#include "slamaterialextensioninterface.h"
#include <fstream>

///@name Input fields for IntMatIsoDamageTableSla
//@{
#define _IFT_IntMatIsoDamageTableSla_Name "intmatisodamagetablesla"
//@}

namespace oofem {

class IntMatIsoDamageTableStatusSla : public IntMatIsoDamageStatus, public SlaMaterialStatusExtensionInterface
{
private:

public:
  /// Constructor
  IntMatIsoDamageTableStatusSla(int n, Domain *d, GaussPoint *g) : IntMatIsoDamageStatus(n,d,g), SlaMaterialStatusExtensionInterface(n,d,g,3,1) {}
  /// Destructor
  virtual ~IntMatIsoDamageTableStatusSla(){}

  // definition
  virtual const char *giveClassName() const { return "IntMatIsoDamageTableStatusSla"; }

  virtual void printOutputAt(FILE *file, TimeStep *tStep);

};

/**
 * Simple isotropic damage based model for 2d and 3d interface elements (- see intmatisodamagetable)
 * adjusted for sequentially linear analysis
 *
 * Differences between this class and IsoInterfaceDamageMaterialTable written by:
 * @author Jan Vorel
 */

class IntMatIsoDamageTableSla : public IntMatIsoDamageTable, public SlaMaterialExtensionInterface
{
protected:
  FloatArray giveJump3D(const FloatArray &jump);
  FloatArray giveJumpReducedForm(const FloatArray &jump3D, const int nDim);

  virtual void give2dStiffnessMatrix_Eng(FloatMatrix &answer, MatResponseMode rMode,
                                              GaussPoint *gp, TimeStep *tStep);
  virtual void give3dStiffnessMatrix_Eng(FloatMatrix &answer, MatResponseMode rMode,
                                              GaussPoint *gp, TimeStep *tStep);

public:
  /// Constructor
  IntMatIsoDamageTableSla(int n, Domain *d);
  /// Destructor
  virtual ~IntMatIsoDamageTableSla(){}

  virtual const char *giveInputRecordName() const { return _IFT_IntMatIsoDamageTableSla_Name; }

  virtual void giveEngTraction_3d(FloatArray &answer, GaussPoint *gp, const FloatArray &jump, TimeStep *tStep);

  /**
     * Computes the value of damage parameter omega, based on given value of equivalent strain.
     * It uses a table (jump vs. damage) and interpolates linearly in between.
     * @param[out] omega Contains result.
     * @param kappa Equivalent strain measure.
     */
  virtual void updateDamageParam(IntMatIsoDamageTableStatusSla *status);

  virtual IRResultType initializeFrom(InputRecord *ir);
  virtual void giveInputRecord(DynamicInputRecord &input);
  virtual MaterialStatus *CreateStatus(GaussPoint *gp) const { return new IntMatIsoDamageTableStatusSla(1, domain, gp); }

  virtual double computeLambda(GaussPoint *gp, FloatArray &totalStrain, TimeStep *tStep, double minLambda = INFINITY);
  virtual void initSLA(GaussPoint *gp, FloatArray &totalStrain, TimeStep *tStep);
  virtual void updateHistoryParameters(GaussPoint *gp, FloatArray &totalStrain, TimeStep *tStep, bool activeDamage);

};
} // end namespace oofem
#endif // isointerfacedamage01_h
